/* jQuery TreeTable Core 2.0 stylesheet
 *
 * This file contains styles that are used to display the tree table. Each tree
 * table is assigned the +treeTable+ class.
 * ========================================================================= */

/* jquery.treeTable.collapsible
 * ------------------------------------------------------------------------- */
.treeTable tr td .expander {
  background-position: left center;
  background-repeat: no-repeat;
  cursor: pointer;
  padding: 0;
  zoom: 1; /* IE7 Hack */
}

.treeTable td {
    vertical-align: top;
    border-right:1px solid #8CACBB;
}

.treeTable tr.odd {
    background-color: buttonface;
}

.treeTable tr.collapsed td .expander {
  background-image: url(++resource++permissionmanager.resources/toggle-expand-dark.png);
  margin-left:-5px;
}


.treeTable tr.expanded td .expander {
  background-image: url(++resource++permissionmanager.resources/toggle-collapse-dark.png);
  margin-left:-5px;
}

/* jquery.treeTable.sortable
 * ------------------------------------------------------------------------- */
.treeTable tr.selected, .treeTable tr.accept {
  background-color: #3875d7;
  color: #fff;
}

.treeTable tr.collapsed.selected td .expander, .treeTable tr.collapsed.accept td .expander {
  background-image: url(++resource++permissionmanager.resources/toggle-expand-light.png);
}

.treeTable tr.expanded.selected td .expander, .treeTable tr.expanded.accept td .expander {
  background-image: url(++resource++permissionmanager.resources/toggle-collapse-light.png);
}

.treeTable .ui-draggable-dragging {
  color: #000;
  z-index: 1;
}

.treeTable tr.notAquired {
    border-top: 2px solid red;
}